/* COPYRIGHT (C) GUPTA TECHNOLOGIES, INC. 1984-1991 */
/*
FUNCTION NAME
  gpimai - General Programming Interface MAI
REVISION HISTORY
  06/30/87 GTI original
  01/21/88 GTI worked around Borland's TurboC memcmp() bug
  03/31/88 GTI addition of SQLNRR, SQLSTR, SQLSPR
  04/14/88 GTI added sqlcnc(), sqlcnr(), sqloms(), sqlims() & sqlcsp().
  05/04/88 GTI changed interface to sqlsto()
  05/31/88 GTI added sqldsc()
  06/28/88 GTI added sqllab()
  08/07/88 GTI added sqlcbv()
  10/07/88 GTI added sqlsrd() and sqlfrs()
  10/15/88 GTI remove server number for sqldin()
  01/30/89 GTI server API
  06/22/89 J.J v4 p1 integration
  10/31/90 JSC bug # 3156 fix: initialize error position before calling sqlepo
  11/22/90 DBS bug # 4204 fix: add Gpires flag
  02/08/91 DWN bug fix: don't change with FP_OFF
EXTERNAL INTERFACE
  FUNCTION
    SQL General Programming Interface to resident database
  DESCRIPTION
    This file implements the general programming  interface  to  the  resident
    database.  It assumes the use of the Microsoft C compiler.	This  function
    is called SQLMAI to insure it won't be included in the library.
  RETURN
INTERNAL DESCRIPTION
  All SQL API functions perform interrupt to resident  database,  passing  the
  argument list in the parameter block.  Since the Microsoft Pascal  interface
  and far pointers are used, this code is not compatible with any C  compiler.
  However, this file could be used as a starting point to interface with any C
  compiler.

  This file can be used to update the prototype declarations by using the  /Zg
  option to the microsoft C compiler.  The functions  SQLSSB  and  SQLDES  are
  located near the beginning of the file to prevent the compiler from crashing
  while using the /Zg option with MSC 4.0.
*/
#define LINT_ARGS
#include "sql.h"
#include "dos.h"
#include "errsql.h"

extern	  int		 Gpires = 1;


/*
  Borland's Turbo C memcmp() has a bug, so this one is provided as a
  substitute.
*/
static int x_memcmp(buf1, buf2, cnt)
char *buf1;
char *buf2;
int cnt;
{
  int diff;

  while (cnt--)
  {
    if ((diff = *buf1++ - *buf2++) != 0)
      return diff;
  }
  return 0;
}

/* int	  _sqlint(int, ...); */
static int	  _sqlino(void);


static	  char		  dnimsg[] = "00140 SQL DNI Database system not installed (start local database or database router).";

#define   P		PTR

/* SQL FUNCTION DEFINITIONS */

/*
 *   sqlarf - SQL Apply Roll Forward journal
 */
SQLTAPI   sqlarf(cur, fnp, fnl, cho)
	  SQLTCUR   cur;		/* -> cursor definition 	    */
	  SQLTFNP   fnp;		/* -> journal filename		    */
	  SQLTFNL   fnl;		/* journal filename length	    */
	  SQLTCHO   cho;		/* cross-check journal w/database   */
{
  return _sqlint(SQLFARF, &cho);
}

/*
 *   sqlbbr - SQL get Bulk Execute Backend error # and message
 */
SQLTAPI   sqlbbr(cur, errno, errbuf, buflen, errrow, rbf, errseq)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTXER P errno;		/* -> return code		    */
	  SQLTDAP   errbuf;		/* -> backend error message	    */
	  SQLTDAL P buflen;		/* -> error msg length		    */
	  SQLTBIR P errrow;		/* -> error row number		    */
	  SQLTRBF P rbf;		/* -> roll back flag		    */
	  SQLTBIR   errseq;		/* error sequence number	    */
{
  return _sqlint(SQLFBBR, &errseq);
}

/*
 *   sqlbdb - SQL Backup DataBase
 */
SQLTAPI   sqlbdb (shandle, dbname, dbnamel, bkpdir, bkpdirl, local, over)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTFNP   bkpdir;		/* pointer to backup directory	    */
	  SQLTFNL   bkpdirl;		/* backup directory length	    */
	  SQLTBOO   local;		/* backup file on client machine?   */
	  SQLTBOO   over;		/* true: overwrite existing file    */
{
  return _sqlint(SQLFBDB, &over);
}

/*
 *   sqlbef - SQL Bulk Execute Flush
 */
SQLTAPI   sqlbef(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFBEF, &cur);
}

/*
 *   sqlber - SQL get Bulk Execute Returned code
 */
SQLTAPI   sqlber(cur, rcd, errrow, rbf, errseq)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTRCD P rcd;		/* -> return code		    */
	  SQLTBIR P errrow;		/* error row number		    */
	  SQLTRBF P rbf;		/* -> roll back flag		    */
	  SQLTBIR   errseq;		/* error sequence number	    */
{
  return _sqlint(SQLFBER, &errseq);
}

/*
 *   sqlbkp - SQL BacKuP
 */
SQLTAPI   sqlbkp(cur, defalt, overwrt, bkfname, bkfnlen)
	  SQLTCUR   cur;		/* current cursor		    */
	  SQLTBOO   defalt;		/* true: backup on same node using the
					   default backup file name	    */
	  SQLTBOO   overwrt;		/* overwrite the backup file if it
					   already exists.		    */
	  SQLTFNP   bkfname;		/* -> backup filename		    */
	  SQLTFNL   bkfnlen;		/* length of filename		    */
{
  return _sqlint(SQLFBKP, &bkfnlen);
}

/*
 *   sqlbld - SQL Bind Long Data variables
 */
SQLTAPI   sqlbld(cur, bnp, bnl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTBNP   bnp;		/* -> name of program variable	    */
	  SQLTBNL   bnl;		/* length of program variable name  */
{
  return _sqlint(SQLFBLD, &bnl);
}

/*
 *   sqlblf - SQL Backup Log Files
 */
SQLTAPI   sqlblf (shandle, dbname, dbnamel, bkpdir, bkpdirl, local, over)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTFNP   bkpdir;		/* pointer to backup directory	    */
	  SQLTFNL   bkpdirl;		/* backup directory length	    */
	  SQLTBOO   local;		/* backup file on client machine?   */
	  SQLTBOO   over;		/* true: overwrite existing file    */
{
  return _sqlint(SQLFBLF, &over);
}

/*
 *   sqlblk - SQL set BuLK insert mode
 */
SQLTAPI   sqlblk(cur, blkflg)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTFLG   blkflg;		/* 0/1 = off/on 		    */
{
  return _sqlint(SQLFBLK, &blkflg);
}

/*
 *   sqlbln - SQL Bind Long Number
 */
SQLTAPI   sqlbln(cur, bnn)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTBNN   bnn;		/* bind number			    */
{
  return _sqlint(SQLFBLN, &bnn);
}

/*
 *   sqlbna - SQL BiNd Data variables by name
 */
SQLTAPI   sqlbna(cur, bnp, bnl, dap, dal, sca, pdt, nli)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTBNP   bnp;		/* -> name of program variable	    */
	  SQLTBNL   bnl;		/* length of program variable name  */
	  SQLTDAP   dap;		/* -> bind data buffer		    */
	  SQLTDAL   dal;		/* bind data length		    */
	  SQLTSCA   sca;		/* scale			    */
	  SQLTPDT   pdt;		/* bind program data type	    */
	  SQLTNUL   nli;		/* null indicator		    */
{
  return _sqlint(SQLFBNA, &nli);
}

/*
 *   sqlbnd - SQL BiNd Data variables.	This function is superceded by SQLBNA
 */
SQLTAPI   sqlbnd(cur, bnp, bnl, dap, dal, sca, pdt)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTBNP   bnp;		/* -> name of program variable	    */
	  SQLTBNL   bnl;		/* length of program variable name  */
	  SQLTDAP   dap;		/* -> bind data buffer		    */
	  SQLTDAL   dal;		/* bind data length		    */
	  SQLTSCA   sca;		/* scale			    */
	  SQLTPDT   pdt;		/* bind program data type	    */
{
  return _sqlint(SQLFBND, &pdt);
}

/*
 *   sqlbnn - SQL BiNd Numerics
 */
SQLTAPI   sqlbnn(cur, bnn, dap, dal, sca, pdt)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTBNN   bnn;		/* bind number			    */
	  SQLTDAP   dap;		/* -> bind data buffer		    */
	  SQLTDAL   dal;		/* bind data length		    */
	  SQLTSCA   sca;		/* scale			    */
	  SQLTPDT   pdt;		/* bind program data type	    */
{
  return _sqlint(SQLFBNN, &pdt);
}

/*
 *   sqlbnu - SQL BiNd Numerical bind variable
 */
SQLTAPI   sqlbnu(cur, bnn, dap, dal, sca, pdt, nli)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTBNN   bnn;		/* bind number			    */
	  SQLTDAP   dap;		/* -> bind data buffer		    */
	  SQLTDAL   dal;		/* bind data length		    */
	  SQLTSCA   sca;		/* scale			    */
	  SQLTPDT   pdt;		/* bind program data type	    */
	  SQLTNUL   nli;		/* null indicator		    */
{
  return _sqlint(SQLFBNU, &nli);
}

/*
 *   sqlbss - SQL Backup SnapShot
 */
SQLTAPI   sqlbss (shandle, dbname, dbnamel, bkpdir, bkpdirl, local, over)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTFNP   bkpdir;		/* pointer to backup directory	    */
	  SQLTFNL   bkpdirl;		/* backup directory length	    */
	  SQLTBOO   local;		/* backup file on client machine?   */
	  SQLTBOO   over;		/* true: overwrite existing file    */
{
  return _sqlint(SQLFBSS, &over);
}

/*
 *   sqlcan - SQL CaNceL command
 */
SQLTAPI   sqlcan(cur)
	  SQLTCUR   cur;		/* -> cursor definition 	    */
{
  return _sqlint(SQLFCAN, &cur);
}

/*
 *   sqlcbv - SQL Clear Bind Variables
 */
SQLTAPI   sqlcbv(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFCBV, &cur);
}

/*
 *   sqlcex - SQL Compile and EXecute a SQL command
 */
SQLTAPI   sqlcex(cur, dap, dal)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   dap;		/* -> command buffer		    */
	  SQLTDAL   dal;		/* command length		    */
{
  return _sqlint(SQLFCEX, &dal);
}

/*
 *   sqlclf - SQL Change server activity LogFile
 */
SQLTAPI   sqlclf(cur, logfile, startflag)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTDAP   logfile;		/* log filename to open 	    */
	  SQLTFMD   startflag;		/* start activity log flag	    */
{
  return _sqlint(SQLFCLF, &startflag);
}

/*
 *   sqlcmt - SQL CoMmiT a transaction to the database
 */
SQLTAPI   sqlcmt(cur)
	  SQLTCUR   cur;		/* -> cursor definition 	    */
{
  return _sqlint(SQLFCMT, &cur);
}

/*
 *   sqlcnc - SQL CoNneCt: the sequel.
 */
SQLTAPI   sqlcnc(curp, dbnamp, dbnaml)
	  SQLTCUR P curp;		/* -> cursor number		    */
	  SQLTDAP   dbnamp;		/* -> database name buffer	    */
	  SQLTDAL   dbnaml;		/* database name length 	    */
{
  return _sqlint(SQLFCNC, &dbnaml);
}

/*
 *   sqlcnr - SQL Connect with No Recovery
 */
SQLTAPI   sqlcnr(curp, dbnamp, dbnaml)
	  SQLTCUR P curp;		/* -> cursor number		    */
	  SQLTDAP   dbnamp;		/* -> database name buffer	    */
	  SQLTDAL   dbnaml;		/* database name length 	    */
{
  return _sqlint(SQLFCNR, &dbnaml);
}

/*
 *   sqlcom - SQL COMpile a SQL command
 */
SQLTAPI   sqlcom(cur, cmdp, cmdl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   cmdp;		/* -> command buffer		    */
	  SQLTDAL   cmdl;		/* command length		    */
{
  return _sqlint(SQLFCOM, &cmdl);
}

/*
 *   sqlcon - SQL CONnect to a cursor/database
 */
SQLTAPI   sqlcon(curp, dbnamp, dbnaml, cursiz, pages, recovr, outsize, insize)
	  SQLTCUR P curp;		/* -> cursor number		    */
	  SQLTDAP   dbnamp;		/* -> database name buffer	    */
	  SQLTDAL   dbnaml;		/* database name length 	    */
	  SQLTWSI   cursiz;		/* cursor work size		    */
	  SQLTNPG   pages;		/* number of cache pages to allocate */
	  SQLTRCF   recovr;		/* recovery flag		    */
	  SQLTDAL   outsize;		/* output message buffer size	    */
	  SQLTDAL   insize;		/* input message buffer size	    */
{
  return _sqlint(SQLFCON, &insize);
}

/*
 *   sqlcpy - SQL CoPY
 */
SQLTAPI   sqlcpy(fcur, selp, sell, tcur, isrtp, isrtl)
	  SQLTCUR   fcur;		/* from cursor			    */
	  SQLTDAP   selp;		/* -> select command		    */
	  SQLTDAL   sell;		/* select command length	    */
	  SQLTCUR   tcur;		/* to cursor			    */
	  SQLTDAP   isrtp;		/* -> insert command		    */
	  SQLTDAL   isrtl;		/* insert command length	    */
{
  return _sqlint(SQLFCPY, &isrtl);
}

/*
 *   sqlcre - SQL CREate database
 */
SQLTAPI   sqlcre(shandle, dbnamp, dbnaml)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbnamp;		/* pointer to database name	    */
	  SQLTDAL   dbnaml;		/* length of database name	    */
{
  return _sqlint(SQLFCRE, &dbnaml);
}

/*
 *   sqlcrf - SQL Continue RollForward recovery
 */
SQLTAPI   sqlcrf(shandle, dbname, dbnamel)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
{
  return _sqlint(SQLFCRF, &dbnamel);
}

/*
 *   sqlcrs - SQL Close Restriction Set
 */
SQLTAPI   sqlcrs(cur, rsp, rsl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   rsp;		/* -> restriction set name buffer   */
	  SQLTDAL   rsl;		/* restriction set name length	    */
{
  return _sqlint(SQLFCRS, &rsl);
}

/*
 *   sqlcsv - SQL Connect to SerVer
 */
SQLTAPI   sqlcsv(handlep, serverid, password)
	  SQLTSVH P handlep;		/* handle number		    */
	  SQLTDAP   serverid;		/* null-terminated server name	    */
	  SQLTDAP   password;		/* null-terminated server password  */
{
  return _sqlint(SQLFCSV, &password);
}

/*
 *   sqlcty - SQL get Command TYpe
 */
SQLTAPI   sqlcty(cur, cty)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTCTY P cty;		/* -> command type		    */
{
  return _sqlint(SQLFCTY, &cty);
}

/*
 *   sqldbn - SQL Directory By Name
 */
SQLTAPI   sqldbn(serverid, buffer, length)
	  SQLTDAP   serverid;		/* null-terminated server id	    */
	  SQLTDAP   buffer;		/* -> directory buffer		    */
	  SQLTDAL   length;		/* buffer length		    */
{
  return _sqlint(SQLFDBN, &length);
}

/*
 *   sqlded - SQL DeINstall database
 */
SQLTAPI   sqlded(shandle, dbnamp, dbnaml)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbnamp;		/* pointer to database name	    */
	  SQLTDAL   dbnaml;		/* length of database name	    */
{
  return _sqlint(SQLFDED, &dbnaml);
}

/*
 *   sqldel - SQL DELete database
 */
SQLTAPI   sqldel(shandle, dbnamp, dbnaml)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbnamp;		/* pointer to database name	    */
	  SQLTDAL   dbnaml;		/* length of database name	    */
{
  return _sqlint(SQLFDEL, &dbnaml);
}

/*
 *   sqldes - SQL DEScribe the items of a select statement
 */
SQLTAPI   sqldes(cur, slc, ddt, ddl, chp, chlp, prep, scap)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column number 	    */
	  SQLTDDT P ddt;		/* -> database data type	    */
	  SQLTDDL P ddl;		/* -> database data length	    */
	  SQLTCHP   chp;		/* -> column heading buffer	    */
	  SQLTCHL P chlp;		/* -> column heading length	    */
	  SQLTPRE P prep;		/* -> numeric precision 	    */
	  SQLTSCA P scap;		/* -> numeric scale		    */
{
  return _sqlint(SQLFDES, &scap);
}

/*
 *   sqldid - SQL DeInstall database and Delete database files
 */
SQLTAPI   sqldid(dbname, dbnamel)
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
{
  return _sqlint(SQLFDID, &dbnamel);
}

/*
 *   sqldin - SQL DeINstall database (no longer supported)
 */
SQLTAPI   sqldin(dbnamp, dbnaml)
	  SQLTDAP   dbnamp;		/* -> database name buffer	    */
	  SQLTDAL   dbnaml;		/* database name length 	    */
{
  return _sqlint(SQLFDIN, &dbnaml);
}

/*
 *   sqldir - SQL DIRectory of databases
 */
SQLTAPI   sqldir(srvno, buffer, length)
	  SQLTSVN   srvno;		/* server number		    */
	  SQLTDAP   buffer;		/* -> directory buffer		    */
	  SQLTDAL   length;		/* buffer length		    */
{
  return _sqlint(SQLFDIR, &length);
}

/*
 *   sqldis - SQL DISconnect from a cursor/database
 */
SQLTAPI   sqldis(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFDIS, &cur);
}

/*
 *   sqldon - SQL DONe using database
 */
SQLTAPI   sqldon (void)
{
  return _sqlint(SQLFDON);
}

/*
 *   sqldrc - SQL DiRectory Close
 */
SQLTAPI   sqldrc(cur)
	  SQLTSVH   cur;		/* server handle		    */
{
  return _sqlint(SQLFDRC, &cur);
}

/*
 *   sqldro - SQL DiRectory Open
 */
SQLTAPI   sqldro(cur, dirname)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTDAP   dirname;		/* directory to open		    */
{
  return _sqlint(SQLFDRO, &dirname);
}

/*
 *   sqldrr - SQL DiRectory Read
 */
SQLTAPI   sqldrr(cur, filename)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTDAP   filename;		/* directory to open		    */
{
  return _sqlint(SQLFDRR, &filename);
}

/*
 *   sqldrs - SQL Drop Restriction Set
 */
SQLTAPI   sqldrs(cur, rsp, rsl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   rsp;		/* -> restriction set name buffer   */
	  SQLTDAL   rsl;		/* restriction set name length	    */
{
  return _sqlint(SQLFDRS, &rsl);
}

/*
 *   sqldsc - SQL DeSCribe item of SELECT with external type
 */
SQLTAPI   sqldsc(cur, slc, edt, edl, chp, chlp, prep, scap)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column number 	    */
	  SQLTDDT P edt;		/* -> external data type	    */
	  SQLTDDL P edl;		/* -> external data length	    */
	  SQLTCHP   chp;		/* -> column heading buffer	    */
	  SQLTCHL P chlp;		/* -> column heading length	    */
	  SQLTPRE P prep;		/* -> numeric precision 	    */
	  SQLTSCA P scap;		/* -> numeric scale		    */
{
  return _sqlint(SQLFDSC, &scap);
}

/*
 *   sqldst - SQL Drop a STored command
 */
SQLTAPI   sqldst(cur, cnp, cnl)
	  SQLTCUR   cur;		/* -> cursor definition 	    */
	  SQLTDAP   cnp;		/* -> command name buffer	    */
	  SQLTDAL   cnl;		/* command name length		    */
{
  return _sqlint(SQLFDST, &cnl);
}

/*
 *   sqldsv - SQL Disconnect from SerVer
 */
SQLTAPI   sqldsv(handle)
	  SQLTSVH   handle;		/* handle number		    */
{
  return _sqlint(SQLFDSV, &handle);
}

/*
 *   sqlebk - SQL End network online BacKup
 */
SQLTAPI   sqlebk(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFEBK, &cur);
}

/*
 *   sqlefb - SQL Enable Fetch Backwards
 */
SQLTAPI   sqlefb(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFEFB, &cur);
}

/*
 *   sqlelo - SQL End Long Operation
 */
SQLTAPI   sqlelo(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFELO, &cur);
}

/*
 *   sqlenr - SQL ENd Rollforward recovery
 */
SQLTAPI   sqlenr(shandle, dbname, dbnamel)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
{
  return _sqlint(SQLFENR, &dbnamel);
}

/*
 *   sqlepo - SQL get Error POsition
 */
SQLTAPI   sqlepo(cur, epo)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTEPO P epo;		/* -> error position		    */
{
  return _sqlint(SQLFEPO, &epo);
}

/*
 *   sqlerf - SQL End RollForward recovery (no longer supported)
 */
SQLTAPI   sqlerf(cur)
	  SQLTCUR   cur;		/* cursor			    */
{
  return _sqlint(SQLFERF, &cur);
}

/*
 *   sqlerr - SQL get ERRor message
 */
SQLTAPI   sqlerr(error, msg)
	  SQLTRCD   error;		/* error number to be looked up     */
	  SQLTDAP   msg;		/* message buffer pointer	    */
{
  error = _sqlint(SQLFERR, &msg);	/* get return value		    */
  if (error == SQLEDNI) 		/* database not installed?	    */
    strcpy(msg, dnimsg + 14);		/* put in the message		    */
  return error; 			/* return the value		    */
}

/*
 *   sqlers - SQL End network ReStore
 */
SQLTAPI   sqlers(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFERS, &cur);
}

/*
 *   sqletx - SQL get Error message TeXt
 */
SQLTAPI   sqletx(error, msgtyp, bfp, bfl, txtlen)
	  SQLTRCD   error;		/* error number to be looked up     */
	  SQLTPTY   msgtyp;		/* error msg text type to retrieve  */
	  SQLTDAP   bfp;		/* msg text buffer pointer	    */
	  SQLTDAL   bfl;		/* msg test buffer length	    */
	  SQLTDAL P txtlen;		/* actual length of the msg	    */
{
  return _sqlint(SQLFETX, &txtlen);
}

/*
 *   sqlexe - SQL EXEcute an SQL command
 */
SQLTAPI   sqlexe(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFEXE, &cur);
}

/*
 *   sqlexp - SQL EXexcution Plan
 */
SQLTAPI   sqlexp(cur, buffer, length)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   buffer;		/* -> execution plan buffer	    */
	  SQLTDAL   length;		/* buffer length		    */
{
  return _sqlint(SQLFEXP, &length);
}

/*
 *   sqlfbk - SQL FETch previous row from SELECT
 */
SQLTAPI   sqlfbk(cur)
	  SQLTCUR   cur;		/* -> cursor definition 	    */
{
  return _sqlint(SQLFFBK, &cur);
}

/*
 *   sqlfer - SQL Full ERror message
 */
SQLTAPI   sqlfer(error, msg)
	  SQLTRCD   error;		/* error number to be looked up     */
	  SQLTDAP   msg;		/* message buffer pointer	    */
{
  error = _sqlint(SQLFFER, &msg);	/* get return value		    */
  if (error == SQLEDNI) 		/* database not installed?	    */
    strcpy(msg, dnimsg);		/* put in the message		    */
  return error; 			/* return the value		    */
}

/*
 *   sqlfet - SQL FETch next row from SELECT
 */
SQLTAPI   sqlfet(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFFET, &cur);
}

/*
 *   sqlfgt - SQL File GeT
 */
SQLTAPI   sqlfgt(cur, srvfile, lclfile)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTDAP   srvfile;		/* server filename		    */
	  SQLTDAP   lclfile;		/* local filename		    */
{
  return _sqlint(SQLFFGT, &lclfile);
}

/*
 *   sqlfpt - SQL File Put
 */
SQLTAPI   sqlfpt(cur, srvfile, lclfile)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTDAP   srvfile;		/* server filename		    */
	  SQLTDAP   lclfile;		/* local filename		    */
{
  return _sqlint(SQLFFPT, &lclfile);
}

/*
 *   sqlfqn - SQL get Fully Qualified column Name
 */
SQLTAPI   sqlfqn(cur, field, nameptr, namelen)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTFLD   field;		/* field number 		    */
	  SQLTDAP   nameptr;		/* -> column name buffer	    */
	  SQLTDAL P namelen;		/* ->length of fully qualified name */
{
  return _sqlint(SQLFFQN, &namelen);
}

/*
 *   sqlgdi - SQL Get Descriptor info for a given Select column.
 */
SQLTAPI   sqlgdi(cur, gdi)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTPGD   gdi;		/* SQL descriptor structure	    */
{
  return _sqlint(SQLFGDI, &gdi);
}

/*
 *   sqlget - SQL GET database parameter
 */
SQLTAPI   sqlget(cur, parm, p, l)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTPTY   parm;		/* paramter type		    */
	  SQLTDAP   p;			/* -> information buffer	    */
	  SQLTDAL P l;			/* -> information length	    */
{
  return _sqlint(SQLFGET, &l);
}

/*
 *   sqlgfi - SQL Get Fetch Information
 */
SQLTAPI   sqlgfi(cur, slc, cvl, fsc)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column		    */
	  SQLTCDL P cvl;		/* -> current value length	    */
	  SQLTFSC P fsc;		/* -> fetch status code 	    */
{
  return _sqlint(SQLFGFI, &fsc);
}

/*
 *   sqlgls - SQL Get Long Size
 */
SQLTAPI   sqlgls(cur, slc, size)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column		    */
	  SQLTLSI P size;		/* -> size			    */
{
  return _sqlint(SQLFGLS, &size);
}

/*
 *   sqlgnl - SQL Get Next Log for rollforward
 */
SQLTAPI   sqlgnl (shandle, dbname, dbnamel, lognum)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTLNG P lognum;		/* returned log number		    */
{
  return _sqlint(SQLFGNL, &lognum);
}

/*
 *   sqlgnr - SQL Get Number of Rows
 */
SQLTAPI   sqlgnr(cur, tbnam, tbnaml, rows)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   tbnam;		/* table name			    */
	  SQLTDAL   tbnaml;		/* table name length		    */
	  SQLTROW P rows;		/* number of rows		    */
{
  return _sqlint(SQLFGNR, &rows);
}

/*
 *   sqlgsi - SQL Get Server Information
 */
SQLTAPI   sqlgsi(cur, infoflags, buffer, buflen, rbuflen)
	  SQLTSVH   cur;		/* handle number		    */
	  SQLTFLG   infoflags;		/* information flags		    */
	  SQLTDAP   buffer;		/* buffer to read into		    */
	  SQLTDAL   buflen;		/* size of buffer		    */
	  SQLTDAL P rbuflen;		/* amount of buffer filled	    */
{
  return _sqlint(SQLFGSI, &rbuflen);
}

/*
 *   sqlidb - SQL Initialize DataBase
 */
SQLTAPI   sqlidb(cur)
	  SQLTCUR   cur;	       /* cursor number 		    */
{
  return _sqlint(SQLFIDB, &cur);
}

/*
 *   sqlims - SQL set Input Message Size
 */
SQLTAPI   sqlims(cur, insize)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAL   insize;		/* input message buffer size	    */
{
  return _sqlint(SQLFIMS, &insize);
}

/*
 *   sqlind - SQL INstall Database
 */
SQLTAPI   sqlind(shandle, dbnamp, dbnaml)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbnamp;		/* pointer to database name	    */
	  SQLTDAL   dbnaml;		/* length of database name	    */
{
  return _sqlint(SQLFIND, &dbnaml);
}

/*
 *   sqlini - SQL INItialize applications use of the database
 */
SQLTAPI   sqlini(callback)
	  SQLTPFP   callback;	       /* callback function		    */
{
  return _sqlint(SQLFINI, &callback);
}

/*
 *   sqlins - SQL INStall database (no longer supported)
 */
SQLTAPI   sqlins(srvno, dbnamp, dbnaml, createflag, overwrite)
	  SQLTSVN   srvno;		/* server number		    */
	  SQLTDAP   dbnamp;		/* -> database name buffer	    */
	  SQLTDAL   dbnaml;		/* database name length 	    */
	  SQLTFLG   createflag; 	/* create flag			    */
	  SQLTFLG   overwrite;		/* overwrite flag		    */
{
  return _sqlint(SQLFINS, &overwrite);
}

/*
 *   sqllab - SQL get LABel information
 */
SQLTAPI   sqllab(cur, slc, lbp, lblp)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column number 	    */
	  SQLTCHP   lbp;		/* -> buffer to retrieve label	    */
	  SQLTCHL P lblp;		/* -> label name length 	    */
{
  return _sqlint(SQLFLAB, &lblp);
}

/*
 *   sqllsk - SQL Long SeeK
 */
SQLTAPI   sqllsk(cur, slc, pos)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column of LONG field	    */
	  SQLTLSI   pos;		/* desired byte position within LONG */
{
  return _sqlint(SQLFLSK, &pos);
}

/*
 *   sqlmcl - SQL reMote CLose server file
 */
SQLTAPI   sqlmcl(cur, fd)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTFLH   fd; 		/* (virtual) file handle	    */
{
  return _sqlint(SQLFMCL, &fd);
}

/*
 *   sqlmdl - SQL reMote DeLete file or directory on remote server
 */
SQLTAPI   sqlmdl(cur, filename)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTDAP   filename;		/* filename			    */
{
  return _sqlint(SQLFMDL, &filename);
}

/*
 *   sqlmop - SQL reMote OPen file on server
 */
SQLTAPI   sqlmop(cur, fdp, filename, openmode)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTFLH P fdp;		/* -> file handle		    */
	  SQLTDAP   filename;		/* filename to open		    */
	  SQLTFMD   openmode;		/* file open mode		    */
{
  return _sqlint(SQLFMOP, &openmode);
}

/*
 *   sqlmrd - SQL reMote ReaD from file on server
 */
SQLTAPI   sqlmrd(cur, fd, buffer, len, rlen)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTFLH   fd; 		/* (virtual) file handle	    */
	  SQLTDAP   buffer;		/* -> write buffer		    */
	  SQLTDAL   len;		/* write length 		    */
	  SQLTDAL P rlen;		/* -> # bytes written		    */
{
  return _sqlint(SQLFMRD, &rlen);
}

/*
 *   sqlmsk - SQL reMote SeeK into file on server
 */
SQLTAPI   sqlmsk(cur, fd, offset, whence, roffset)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTFLH   fd; 		/* (virtual) file handle	    */
	  SQLTLNG   offset;		/* seek offset			    */
	  SQLTWNC   whence;		/* seek origin			    */
	  SQLTLNG P roffset;		/* result offset		    */
{
  return _sqlint(SQLFMSK, &roffset);
}

/*
 *   sqlmwr - SQL reMote WRite to file on server
 */
SQLTAPI   sqlmwr(cur, fd, buffer, len, rlen)
	  SQLTSVH   cur;		/* server handle		    */
	  SQLTFLH   fd; 		/* (virtual) file handle	    */
	  SQLTDAP   buffer;		/* -> write buffer		    */
	  SQLTDAL   len;		/* write length 		    */
	  SQLTDAL P rlen;		/* -> # bytes written		    */
{
  return _sqlint(SQLFMWR, &rlen);
}

/*
 *   sqlnbv - SQL get Number of Bind Variables
 */
SQLTAPI   sqlnbv(cur, nbv)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTNBV P nbv;		/* -> number of bind variables	    */
{
  return _sqlint(SQLFNBV, &nbv);
}

/*
 *   sqlnrr - SQL Number of Rows in Result set
 */
SQLTAPI   sqlnrr(cur, rcountp)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTROW P rcountp;		/* -> number of result set rows     */
{
  return _sqlint(SQLFNRR, &rcountp);
}

/*
 *   sqlnsi - SQL get Number of Select Items
 */
SQLTAPI   sqlnsi(cur, nsi)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTNSI P nsi;		/* -> number of select items	    */
{
  return _sqlint(SQLFNSI, &nsi);
}

/*
 *   sqloms - SQL set Output Message Size
 */
SQLTAPI   sqloms(cur, outsize)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAL   outsize;		/* output message buffer size	    */
{
  return _sqlint(SQLFOMS, &outsize);
}

/*
 *   sqlprs - SQL Position in Result Set
 */
SQLTAPI   sqlprs(cur, row)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTROW   row;		/* row number to which to position  */
{
  return _sqlint(SQLFPRS, &row);
}

/*
 *   sqlrbf - SQL get Roll Back Flag
 */
SQLTAPI   sqlrbf(cur, rbf)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTRBF P rbf;		/* -> roll back flag		    */
{
  return _sqlint(SQLFRBF, &rbf);
}

/*
 *   sqlrbk - SQL RollBacK a transaction from the database
 */
SQLTAPI   sqlrbk(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFRBK, &cur);
}

/*
 *   sqlrcd - SQL get Return CoDe
 */
SQLTAPI   sqlrcd(cur, rcd)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTRCD P rcd;		/* -> return code		    */
{
  return _sqlint(SQLFRCD, &rcd);
}

/*
 *   sqlrdb - SQL Restore DataBase
 */
SQLTAPI   sqlrdb (shandle, dbname, dbnamel, bkpdir, bkpdirl, local, over)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTFNP   bkpdir;		/* pointer to backup directory	    */
	  SQLTFNL   bkpdirl;		/* backup directory length	    */
	  SQLTBOO   local;		/* backup file on client machine?   */
	  SQLTBOO   over;		/* true: overwrite existing file    */
{
  return _sqlint(SQLFRDB, &over);
}

/*
 *   sqlrdc - SQL Read Database Chunk
 */
SQLTAPI   sqlrdc(cur, bufp, bufl, readl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   bufp;		/* -> read data buffer		    */
	  SQLTDAL   bufl;		/* read data buffer length	    */
	  SQLTDAL P readl;		/* actual length of data read	    */
{
  return _sqlint(SQLFRDC, &readl);
}

/*
 *   sqlrel - SQL RELease log
 */
SQLTAPI   sqlrel(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFREL, &cur);
}

/*
 *   sqlres - SQL REStore from backup
 */
SQLTAPI   sqlres(curptr, bkfname, bkfnlen, bkfserv, overwrt, dbname, dbnlen, dbserv)
	  SQLTCUR P curptr;		/* -> cursor number output argument */
	  SQLTFNP   bkfname;		/* -> backup filename		    */
	  SQLTFNL   bkfnlen;		/* length of filename		    */
	  SQLTSVN   bkfserv;		/* server number where backup is
					   located, 0=local node	    */
	  SQLTBOO   overwrt;		/* if true, overwrite existing database */
	  SQLTDAP   dbname;		/* -> database name		    */
	  SQLTDAL   dbnlen;		/* length of database name	    */
	  SQLTSVN   dbserv;		/* server number where restored     */
{
  return _sqlint(SQLFRES, &dbserv);
}

/*
 *   sqlret - SQL RETrieve a compiled SQL command
 */
SQLTAPI   sqlret(cur, cnp, cnl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   cnp;		/* -> command name buffer	    */
	  SQLTDAL   cnl;		/* command name length		    */
{
  return _sqlint(SQLFRET, &cnl);
}

/*
 *   sqlrlf - SQL Restore Log Files
 */
SQLTAPI   sqlrlf (shandle, dbname, dbnamel, bkpdir, bkpdirl, local, over)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTFNP   bkpdir;		/* pointer to backup directory	    */
	  SQLTFNL   bkpdirl;		/* backup directory length	    */
	  SQLTBOO   local;		/* backup file on client machine?   */
	  SQLTBOO   over;		/* true: overwrite existing file    */
{
  return _sqlint(SQLFRLF, &over);
}

/*
 *   sqlrlo - SQL Read LOng
 */
SQLTAPI   sqlrlo(cur, slc, bufp, bufl, readl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column		    */
	  SQLTDAP   bufp;		/* -> long data buffer		    */
	  SQLTDAL   bufl;		/* long data buffer length	    */
	  SQLTDAL P readl;		/* length of data read		    */
{
  return _sqlint(SQLFRLO, &readl);
}

/*
 *   sqlrof - SQL ROllForward recovery
 */
SQLTAPI   sqlrof(shandle, dbname, dbnamel, mode, datetime, datetimel)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTRFM   mode;		/* rollforward mode		    */
	  SQLTDAP   datetime;		/* -> datetime "mm/dd/yy hh:mm:ss"  */
	  SQLTDAL   datetimel;		/* length of datetime		    */
{
  return _sqlint(SQLFROF, &datetimel);
}

/*
 *   sqlrow - SQL get number of ROWs
 */
SQLTAPI   sqlrow(cur, row)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTROW P row;		/* -> number of rows		    */
{
  return _sqlint(SQLFROW, &row);
}

/*
 *   sqlrrd - SQL Recover Restored Database
 */
SQLTAPI   sqlrrd(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFRRD, &cur);
}

/*
 *   sqlrrs - SQL Restart Restriction Set processing
 */
SQLTAPI   sqlrrs(cur, rsp, rsl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   rsp;		/* -> restriction set name buffer   */
	  SQLTDAL   rsl;		/* restriction set name length	    */
{
  return _sqlint(SQLFRRS, &rsl);
}

/*
 *   sqlrss - SQL Restore SnapShot
 */
SQLTAPI   sqlrss (shandle, dbname, dbnamel, bkpdir, bkpdirl, local, over)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTDAP   dbname;		/* pointer to database name	    */
	  SQLTDAL   dbnamel;		/* length of database name	    */
	  SQLTFNP   bkpdir;		/* pointer to backup directory	    */
	  SQLTFNL   bkpdirl;		/* backup directory length	    */
	  SQLTBOO   local;		/* backup file on client machine?   */
	  SQLTBOO   over;		/* true: overwrite existing file    */
{
  return _sqlint(SQLFRSS, &over);
}

/*
 *   sqlsab - SQL Server ABort process
 */
SQLTAPI   sqlsab(shandle, pnum)
	  SQLTSVH   shandle;		/* server connect handle	    */
	  SQLTPNM   pnum;		/* server process number	    */
{
  return _sqlint(SQLFSAB, &pnum);
}

/*
 *   sqlsap - SQL Server Abort Process
 */
SQLTAPI   sqlsap(srvno, password, pnum)
	  SQLTSVN   srvno;		/* server number		    */
	  SQLTDAP   password;		/* password			    */
	  SQLTPNM   pnum;		/* server process number	    */
{
  return _sqlint(SQLFSAP, &pnum);
}

/*
 *   sqlscn - SQL Set Cursor Name
 */
SQLTAPI   sqlscn(cur, namp, naml)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   namp;		/* -> name buffer		    */
	  SQLTDAL   naml;		/* length of name buffer, or 0	    */
{
  return _sqlint(SQLFSCN, &naml);
}

/*
 *   sqlscp - SQL Set Cache Pages
 */
SQLTAPI   sqlscp(pages)
	  SQLTNPG   pages;		/* number of cache pages to allocate */
{
  return _sqlint(SQLFSCP, &pages);
}

/*
 *   sqlsdn - SQL ShutDowN database
 */
SQLTAPI   sqlsdn(dbnamp, dbnaml)
	  SQLTDAP   dbnamp;		/* -> database name buffer	    */
	  SQLTDAL   dbnaml;		/* database name length 	    */
{
  return _sqlint(SQLFSDN, &dbnaml);
}

/*
 *   sqlset - SQL SET database parameter
 */
SQLTAPI   sqlset(cur, parm, p, l)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTPTY   parm;		/* paramter type		    */
	  SQLTDAP   p;			/* -> information buffer	    */
	  SQLTDAL   l;			/* information length		    */
{
  return _sqlint(SQLFSET, &l);
}

/*
 *   sqlsil - SQL Set Isolation Level
 */
SQLTAPI   sqlsil(cur, isolation)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTILV   isolation;		/* desired isolation level	    */
{
  return _sqlint(SQLFSIL, &isolation);
}

/*
 *   sqlslp - SQL Set Log Parameters
 */
SQLTAPI   sqlslp(cur, lpt, lpm)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTNPG   lpt;		/* log page threshold		    */
	  SQLTNPG   lpm;		/* log page maximum		    */
{
  return _sqlint(SQLFSLP, &lpm);
}

/*
 *   sqlspr - SQL StoP Restriction mode
 */
SQLTAPI   sqlspr(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFSPR, &cur);
}

/*
 *   sqlsrf - SQL Start Roll Forward journal
 */
SQLTAPI   sqlsrf(cur, fnp, fnl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   fnp;		/* -> journal filename		    */
	  SQLTDAL   fnl;		/* journal filename length	    */
{
  return _sqlint(SQLFSRF, &fnl);
}

/*
 *   sqlsrs - SQL Start Restriction Set processing
 */
SQLTAPI   sqlsrs(cur)
	  SQLTCUR   cur;		/* -> cursor definition 	    */
{
  return _sqlint(SQLFSRS, &cur);
}

/*
 *   sqlssb - SQL Set Select Buffer
 */
SQLTAPI   sqlssb(cur, slc, pdt, pbp, pdl, sca, pcv, pfc)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSLC   slc;		/* select column		    */
	  SQLTPDT   pdt;		/* program data type		    */
	  SQLTDAP   pbp;		/* -> program buffer		    */
	  SQLTPDL   pdl;		/* program buffer length	    */
	  SQLTSCA   sca;		/* scale			    */
	  SQLTCDL P pcv;		/* -> current value length	    */
	  SQLTFSC P pfc;		/* -> fetch status code 	    */
{
  return _sqlint(SQLFSSB, &pfc);
}

/*
 *   sqlsss - SQL Set SortSpace
 */
SQLTAPI   sqlsss(cur, size)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAL   size;		/* size of sort space		    */
{
  return _sqlint(SQLFSSS, &size);
}

/*
 *   sqlsta - SQL STAtistics
 */
SQLTAPI   sqlsta(cur, svr, svw, spr, spw)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSTC P svr;		/* virtual reads		    */
	  SQLTSTC P svw;		/* virtual writes		    */
	  SQLTSTC P spr;		/* physical reads		    */
	  SQLTSTC P spw;		/* physical writes		    */
{
  return _sqlint(SQLFSTA, &spw);
}

/*
 *   sqlstm - SQL Server TerMinate
 */
SQLTAPI   sqlstm(shandle)
	  SQLTSVH   shandle;		/* server connect handle	    */
{
  return _sqlint(SQLFSTM, &shandle);
}

/*
 *   sqlsto - SQL STOre a compiled SQL command
 */
SQLTAPI   sqlsto(cur, cnp, cnl, ctp, ctl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   cnp;		/* -> command name buffer	    */
	  SQLTDAL   cnl;		/* command name length		    */
	  SQLTDAP   ctp;		/* -> command text buffer	    */
	  SQLTDAL   ctl;		/* command text length		    */
{
  return _sqlint(SQLFSTO, &ctl);
}

/*
 *   sqlstr - SQL STart Restriction mode
 */
SQLTAPI   sqlstr(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFSTR, &cur);
}

/*
 *   sqlsxt - SQL Server eXiT (no longer supported)
 */
SQLTAPI   sqlsxt(srvno, password)
	  SQLTSVN   srvno;		/* server number		    */
	  SQLTDAP   password;		/* password			    */
{
  return _sqlint(SQLFSXT, &password);
}

/*
 *   sqlsys - SQL SYSTEM
 */
SQLTAPI   sqlsys(cur, sys)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTSYS P sys;		/* -> system information	    */
{
  return _sqlint(SQLFSYS, &sys);
}

/*
 *   sqltec - SQL Translate Error Code
 */
SQLTAPI   sqltec(rcd, np)
	  SQLTRCD   rcd;		/* old return code to be converted  */
	  SQLTRCD P np; 		/* -> newly converted return code   */
{
  return _sqlint(SQLFTEC, &np);
}

/*
 *   sqltio - SQL TImeOut
 */
SQLTAPI   sqltio(cur, _timeout)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTTIV   _timeout;		/* timeout value in seconds	    */
{
  return _sqlint(SQLFTIO, &_timeout);
}

/*
 *   sqlurs - SQL Undo Result Set
 */
SQLTAPI   sqlurs(cur)
	  SQLTCUR   cur;		/* cursor number		    */
{
  return _sqlint(SQLFURS, &cur);
}

/*
 *   sqlwdc - SQL Write Database Chunk
 */
SQLTAPI   sqlwdc(cur, bufp, bufl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   bufp;		/* -> long data buffer		    */
	  SQLTDAL   bufl;		/* length of data to write	    */
{
  return _sqlint(SQLFWDC, &bufl);
}

/*
 *   sqlwlo - SQL Write LOng
 */
SQLTAPI   sqlwlo(cur, bufp, bufl)
	  SQLTCUR   cur;		/* cursor number		    */
	  SQLTDAP   bufp;		/* -> long data buffer		    */
	  SQLTDAL   bufl;		/* length of data to write	    */
{
  return _sqlint(SQLFWLO, &bufl);
}

/*
 *   sqlxad - SQL eXtra ADd
 */
SQLTAPI   sqlxad(op, np1, nl1, np2, nl2)
	  SQLTNMP   op; 		/* output pointer		    */
	  SQLTNMP   np1;		/* first number pointer 	    */
	  SQLTNML   nl1;		/* first number length		    */
	  SQLTNMP   np2;		/* second number pointer	    */
	  SQLTNML   nl2;		/* second number length 	    */
{
  return _sqlint(SQLFXAD, &nl2);
}

/*
 *   salxcn - SQL eXtra Character to NUmber
 */
SQLTAPI   sqlxcn(op, ip, il)
	  SQLTNMP   op; 		/* output pointer		    */
	  SQLTDAP   ip; 		/* input pointer		    */
	  SQLTDAL   il; 		/* input length 		    */
{
  return _sqlint(SQLFXCN, &il);
}

/*
 *   sqlxda - SQL eXtra Date Add
 */
SQLTAPI   sqlxda(op, dp, dl, days)
	  SQLTNMP   op; 		/* output pointer		    */
	  SQLTNMP   dp; 		/* input date pointer		    */
	  SQLTNML   dl; 		/* input date length		    */
	  SQLTDAY   days;		/* number of days		    */
{
  return _sqlint(SQLFXDA, &days);
}

/*
 *   sqlxdp - SQL eXtra convert SQLBASE Date to Picture
 */
SQLTAPI   sqlxdp(op, ol, ip, il, pp, pl)
	  SQLTDAP   op; 		/* -> converted internal date	    */
	  SQLTDAL   ol; 		/* converted internal date length   */
	  SQLTNMP   ip; 		/* -> internal SQLBASE date	    */
	  SQLTNML   il; 		/* internal SQLBASE date length     */
	  SQLTDAP   pp; 		/* -> picture string		    */
	  SQLTDAL   pl; 		/* picture string length	    */
{
  return _sqlint(SQLFXDP, &pl);
}

/*
 *   sqlxdv - SQL eXtra DiVide
 */
SQLTAPI   sqlxdv(op, np1, nl1, np2, nl2)
	  SQLTNMP   op; 		/* output pointer		    */
	  SQLTNMP   np1;		/* first number pointer 	    */
	  SQLTNML   nl1;		/* first number length		    */
	  SQLTNMP   np2;		/* second number pointer	    */
	  SQLTNML   nl2;		/* second number length 	    */
{
  return _sqlint(SQLFXDV, &nl2);
}

/*
 *   sqlxer - SQL get backend (Extended) error number and Message
 */
SQLTAPI   sqlxer(cur, errno, errbuf, buflen)
	  SQLTCUR      cur;		/* cursor number		    */
	  SQLTXER P    errno;		/* -> error number		    */
	  SQLTDAP      errbuf;		/* -> error message		    */
	  SQLTDAL P    buflen;		/* -> error message length	    */
{
  return _sqlint(SQLFXER, &buflen);
}

/*
 *   sqlxml - SQL eXtra MuLtiply
 */
SQLTAPI   sqlxml(op, np1, nl1, np2, nl2)
	  SQLTNMP   op; 		/* output pointer		    */
	  SQLTNMP   np1;		/* first number pointer 	    */
	  SQLTNML   nl1;		/* first number length		    */
	  SQLTNMP   np2;		/* second number pointer	    */
	  SQLTNML   nl2;		/* second number length 	    */
{
  return _sqlint(SQLFXML, &nl2);
}

/*
 *   sqlxnp - SQL eXtra convert SQLBASE Numeric to Picture
 */
SQLTAPI   sqlxnp(outp, outl, isnp, isnl, picp, picl)
	  SQLTDAP   outp;		/* -> converted internal number     */
	  SQLTDAL   outl;		/* output buffer length 	    */
	  SQLTNMP   isnp;		/* -> internal SQLBASE number	    */
	  SQLTNML   isnl;		/* internal SQLBASE number length   */
	  SQLTDAP   picp;		/* -> picture specification	    */
	  SQLTDAL   picl;		/* picture specification length     */
{
  return _sqlint(SQLFXNP, &picl);
}

/*
 *   sqlxpd - SQL eXtra convert Picture to SQLBASE Date.
 */
SQLTAPI   sqlxpd(op, olp, ip, pp, pl)
	  SQLTNMP   op; 		/* -> internal SQLBASE date	    */
	  SQLTNML P olp;		/* -> output length		    */
	  SQLTDAP   ip; 		/* -> char string to be converted   */
	  SQLTDAP   pp; 		/* -> picture string		    */
	  SQLTDAL   pl; 		/* picture length		    */
{
  return _sqlint(SQLFXPD, &pl);
}

/*
 *   sqlxsb - SQL eXtra SuBtract
 */
SQLTAPI   sqlxsb(op, np1, nl1, np2, nl2)
	  SQLTNMP   op; 		/* output pointer		    */
	  SQLTNMP   np1;		/* first number pointer 	    */
	  SQLTNML   nl1;		/* first number length		    */
	  SQLTNMP   np2;		/* second number pointer	    */
	  SQLTNML   nl2;		/* second number length 	    */
{
  return _sqlint(SQLFXSB, &nl2);
}

/* END OF SQL FUNCTION DEFINTIONS */

/*
 * _sqlint - Perform interrupt to database
 */
int	  _sqlint(fcode, parms)
	  int	    fcode;
	  char*     parms;
{
	  union     REGS reg;
	  struct    SREGS sreg;
	  unsigned  short parmbuf[16];
	  char P    farp;
static	  int		 intno = 0;		/* interrupt number to database */

  if (!intno)				/* interrupt number not set AND     */
  {
    if (!(intno = _sqlino()))		/* database installed?		    */
      return SQLEDNI;			/* error: database not installed    */
    segread(&sreg);
    reg.x.ax  = 0x8001; 		/* fetch SQL database version	    */
    int86x(intno, &reg, &reg, &sreg);	/* interrupt to SQL database code   */
    if (reg.x.ax != SQLAPIV)		/* API version number match?	    */
    {
      intno = 0;			/* don't remember the interrupt #   */
      return DBOEIVR;			/* error: invalid version	    */
    }
  }

  parmbuf[0] = fcode;

  if (parms)				/* any parameters?		    */
  {
    memcpy(&parmbuf[2], parms,		/* copy parameters to param block   */
	sizeof(parmbuf) -
	(sizeof(parmbuf[0]) * 2));
  }
  farp = (char P) parmbuf;		/* far pointer to parameter block   */
  segread(&sreg);
  reg.x.bx  = FP_OFF(farp);		/* set pointer to parameter block   */
  sreg.ds   = FP_SEG(farp);
  reg.x.ax  = 0x8000;			/* access SQL database		    */
  int86x(intno, &reg, &reg, &sreg);	/* interrupt to SQL database code   */
  return parmbuf[1];			/* return result code		    */
}

/*
 * _sqlino - Locate the installed database interrupt number
 *		Return zero if database is not installed
 */
int	  _sqlino()
{
	  int	    intno;
	  char P    src;
	  char P    dest;
	  int	    len;
static	  char		  magicstr[] = "GTI SQL";
	  char	    buffer[sizeof(magicstr)];
	  char P P  vector;

  vector = 0;
  dest = buffer;
  for (intno = 1; intno < 0x100; intno++)
  {
    src = vector[intno];
    if (src != 0  && FP_OFF(src) >= sizeof(magicstr))
    {
      movedata(FP_SEG(src),
	  FP_OFF(src) - sizeof(magicstr),
	  FP_SEG(dest), FP_OFF(dest),
	  sizeof(buffer));
      if (!x_memcmp(buffer, magicstr, sizeof(magicstr)))
	return intno;
    }
  }
  return 0;				/* not installed		    */
}

/* END OF GPI.C */
